import { PrismaRepository } from '../adapters/secondaries/prisma-repository.abstract';
import { ICollection } from '../interfaces/collection.interface';
import { Point } from './point.type';
import { v4 as uuidv4 } from 'uuid';
import { Territory } from 'src/modules/territories/domain/entities/territory';
import { DatabaseException } from '../exceptions/database.exception';

export class TerritoryRepository<T> extends PrismaRepository<T> {
  async findForPoint(point: Point): Promise<ICollection<T>> {
    return await this.findAllByQuery(
      ['uuid', 'name'],
      [
        `ST_Intersects(ST_GeomFromText('POINT(${point.lon} ${point.lat})',4326),shape) = true`,
      ],
    );
  }

  async createTerritory(territory: Territory): Promise<T> {
    try {
      const affectedRowNumber = await this.createWithFields({
        uuid: `'${uuidv4()}'`,
        name: `'${territory.name}'`,
        shape: `ST_GeomFromGeoJSON('{"type":"MultiPolygon","coordinates":${territory.shape}}')`,
      });
      if (affectedRowNumber == 1) {
        return this.findOne({
          name: territory.name,
        });
      }
      throw new DatabaseException();
    } catch (e) {
      throw e;
    }
  }
}
